@@ -19,6 +19,11 @@
 #'   intercept variable, e.g. "var1", (automatically convered to "(1 | var1)");
 #'   or, (2) the full \code{lme4} specification, e.g. "(var1 | var2)". Note
 #'   parenthesis MUST be included in (2) but NOT included in (1).
+#' @param formula an object of class "formula" (or one that can be coerced to 
+#'   that class). Optional instead of standard dependent/explanatory format. 
+#'   Do not include if using dependent/explanatory. 
+#' @param model_args List. A list of arguments to pass to 
+#' \code{\link{lm}}, \code{\link{glm}}, \code{\link{coxph}}.  
 #' @param column Logical: Compute margins by column rather than row.
 #' @param keep_models Logical: include full multivariable model in output when
 #'   working with reduced multivariable model (\code{explanatory_multi}) and/or
@@ -125,20 +130,36 @@
 #'   finalfit_merge(example.multivariable, last_merge = TRUE)
 #' # finalfit_merge(example.multilevel)
 
-finalfit = function(.data, dependent, explanatory, explanatory_multi=NULL, random_effect=NULL,
+finalfit = function(.data, dependent = NULL, explanatory = NULL, explanatory_multi=NULL, random_effect=NULL,
+										formula = NULL, model_args = list(), 
 										column=NULL, keep_models=FALSE, metrics=FALSE, add_dependent_label=TRUE,
 										dependent_label_prefix="Dependent: ", dependent_label_suffix="", 
 										keep_fit_id = FALSE, ...){
-	if(is.data.frame(.data)==FALSE) stop(".data is not dataframe")
+	
+	# Formula interface -----------------
+	## Added at request
+	if(!is.null(formula) & (!is.null(dependent) | !is.null(explanatory))) stop("Formula OR dependent/explanatory terms, not both")
+	if(!is.null(formula)){
+		.terms = ff_parse_formula(formula)
+		dependent = .terms$dependent
+		explanatory = .terms$explanatory
+		if("random_effect" %in% names(.terms)) random_effect = .terms$random_effect
+	}
+	
+	# Checks
+	if(!is.data.frame(.data)) stop(".data is not dataframe")
 	if(is.null(explanatory)) stop("No explanatory variable(s) provided")
 	if(is.null(dependent)) stop("No dependent variable provided")
 	if(.data[ ,names(.data) %in% explanatory] %>% error_colon_fct_levels()){
 		stop("Colons (:) not allowed in factor-level names. Check with ff_glimpse(.data). Recode: forcats::fct_recode()")
 	}
-
+	if(keep_models & (is.null(random_effect) & is.null(explanatory_multi))){
+		warning("If keep_models = TRUE, explanatory_multi or random_effect must not be null.")
+	}
+	
 	# Fix tibble issue
 	if(any(class(.data) %in% c("tbl_df", "tbl")))  .data = data.frame(.data)
-
+	
 	# What is dependent variable
 	d_variable = .data[,names(.data) %in% dependent]
 	d_is.factor = is.factor(d_variable) |
@@ -153,11 +174,11 @@ finalfit = function(.data, dependent, explanatory, explanatory_multi=NULL, rando
 			column = FALSE
 		}
 	}
-
+	
 	# Arguments to send
 	args = list(.data=.data, dependent=dependent, explanatory=explanatory,
 							explanatory_multi=explanatory_multi,
-							random_effect=random_effect, column = column,
+							random_effect=random_effect, model_args = model_args, column = column,
 							keep_models=keep_models,
 							metrics=metrics,
 							add_dependent_label = add_dependent_label,
@@ -183,140 +204,102 @@ finalfit = function(.data, dependent, explanatory, explanatory_multi=NULL, rando
 #' @rdname finalfit
 #' @export
 finalfit.lm = function(.data, dependent, explanatory, explanatory_multi=NULL, random_effect=NULL,
-											 column = FALSE, keep_models=FALSE, metrics=FALSE, add_dependent_label = TRUE,
+											 model_args, column = FALSE, keep_models=FALSE, metrics=FALSE, add_dependent_label = TRUE,
 											 dependent_label_prefix="Dependent: ", dependent_label_suffix="", 
 											 keep_fit_id=FALSE, ...){
-
+	
 	args = list(...)
-
+	model_args_in = model_args
+	
+	# Define arguments
+	model_args = c(list(.data=.data, dependent=dependent), model_args_in)
+	if(!is.null(random_effect)) model_args = c(model_args, list(random_effect=random_effect)) 
+	if(!is.null(explanatory_multi)){
+		model_args = c(model_args, list(explanatory=explanatory_multi))
+	} else {
+		model_args = c(model_args, list(explanatory=explanatory))
+	}
+	
 	# Defaults which can be modified via ...
-	if (is.null(args$estimate_name)) args$estimate_name = "Coefficient"
-	if (is.null(args$confint_sep)) args$confint_sep = " to "
-
-	# Linear regression model ------------------------------------------
+	if (is.null(args$estimate_name)) args$estimate_name = "OR"
+	
+	# Logistic regression ----
 	# Summary table
-	summary.out = suppressWarnings(
-		summary_factorlist(.data, dependent, explanatory, p=FALSE, na_include=FALSE,
-											 column=column, total_col=FALSE, orderbytotal=FALSE, fit_id=TRUE)
-	)
-
+	summary.out = summary_factorlist(.data, dependent, explanatory, p=FALSE, na_include=FALSE,
+																	 column=column, total_col=FALSE, orderbytotal=FALSE, fit_id=TRUE)
+	
 	# Univariable
-	lmuni.out = lmuni(.data, dependent, explanatory)
+	lmuni.out = do.call(lmuni, c(list(.data=.data, dependent=dependent, explanatory=explanatory), model_args_in))
 	lmuni.df = do.call(fit2df, c(list(.data=lmuni.out, estimate_suffix = " (univariable)"), args))
-
+	
 	# Merge dataframes
 	# Uni
 	df.out = finalfit_merge(summary.out, lmuni.df, estimate_name = args$estimate_name)
-
-	# This is standard approach, which is why written out in full but duplicated somewhat below
-	## Come back and reduce the conditional logic here.
-	## Fast though, as only required models are being made.
-	if(keep_models == FALSE){
-		# Multivariable/Mixed
-		if (is.null(random_effect)){
-			if (is.null(explanatory_multi)){
-				lmmulti.out = lmmulti(.data, dependent, explanatory)
-			} else {
-				lmmulti.out = lmmulti(.data, dependent, explanatory_multi)
-			}
-			lmmulti.df = do.call(fit2df,
-													 c(list(.data=lmmulti.out, metrics=metrics, estimate_suffix = " (multivariable)"), args))
-		} else if (is.null(random_effect) == FALSE){
-			if (is.null(explanatory_multi)){
-				lmmulti.out = lmmixed(.data, dependent, explanatory, random_effect)
-			} else {
-				lmmulti.out = lmmixed(.data, dependent, explanatory_multi, random_effect)
-			}
-			lmmulti.df = do.call(fit2df,
-													 c(list(.data=lmmulti.out, metrics=metrics, estimate_suffix = " (multilevel)"), args))
-		}
-
-		# Multi
-		if (metrics == FALSE){
+	
+	# Multivariable/Mixed
+	if(is.null(random_effect)){
+		lmmulti.out = do.call(lmmulti, model_args)
+		lmmulti.df = do.call(fit2df,
+												 c(list(.data=lmmulti.out, metrics=metrics, estimate_suffix = " (multivariable)"), args))
+	} else if(!is.null(random_effect)){
+		lmmulti.out = do.call(lmmixed, model_args)
+		lmmulti.df = do.call(fit2df,
+												 c(list(.data=lmmulti.out, metrics=metrics, estimate_suffix = " (multilevel)"), args))
+	}
+	
+	# Single multivariable model table
+	if(!keep_models){
+		if(!metrics){
 			df.out = finalfit_merge(df.out, lmmulti.df, estimate_name = args$estimate_name)
 		} else {
 			df.out = finalfit_merge(df.out, lmmulti.df[[1]], estimate_name = args$estimate_name)
 			df.metrics = lmmulti.df[[2]]
 		}
-
-	}else if(keep_models == TRUE){
-		lmmulti1.out = lmmulti(.data, dependent, explanatory)
+	} else if(keep_models){
+		
+		# Keep intermediate models
+		lmmulti1.out = do.call(lmmulti, c(list(.data=.data, dependent=dependent, explanatory=explanatory), model_args_in))
 		lmmulti1.df = do.call(fit2df,
-													c(list(.data=lmmulti1.out, metrics=metrics,
-																 estimate_suffix = " (multivariable)"), args))
-		if (!is.null(explanatory_multi)){
-			lmmulti2.out = lmmulti(.data, dependent, explanatory_multi)
+													c(list(.data=lmmulti1.out, metrics=metrics, estimate_suffix = " (multivariable full)"), args))
+		if(!is.null(random_effect) & !is.null(explanatory_multi)){
+			lmmulti2.out = do.call(
+				lmmixed, c(list(.data=.data, dependent=dependent, explanatory=explanatory, random_effect=random_effect), model_args_in))
 			lmmulti2.df = do.call(fit2df,
-														c(list(.data=lmmulti2.out, metrics=metrics,
-																	 estimate_suffix = " (multivariable reduced)"), args))
+														c(list(.data=lmmulti2.out, metrics=metrics, estimate_suffix = " (multilevel full)"), args))
 		}
-		if (!is.null(random_effect)){
-			if (is.null(explanatory_multi)){
-				lmmixed.out = lmmixed(.data, dependent, explanatory, random_effect)
-			} else {
-				lmmixed.out = lmmixed(.data, dependent, explanatory_multi, random_effect)
-			}
-			lmmixed.df = do.call(fit2df,
-													 c(list(.data=lmmixed.out, metrics=metrics,
-													 			 estimate_suffix = " (multilevel)"), args))
-		}
-
-		if (metrics == FALSE){
-			if (is.null(random_effect)){
-				if (is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(lmmulti1.df, estimate_name = args$estimate_name)
-				}else if(!is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(lmmulti1.df, estimate_name = args$estimate_name) %>%
-						finalfit_merge(lmmulti2.df, estimate_name = args$estimate_name)
+		
+		if(!metrics){
+			df.out = df.out %>%
+				finalfit_merge(lmmulti1.df, estimate_name = args$estimate_name) %>%
+				{ if(!is.null(random_effect) & !is.null(explanatory_multi)){
+					finalfit_merge(., lmmulti2.df, estimate_name = args$estimate_name)
+				} else {
+					.
 				}
-			}else if(!is.null(random_effect)){
-				if (is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(lmmulti1.df, estimate_name = args$estimate_name) %>%
-						finalfit_merge(lmmixed.df, estimate_name = args$estimate_name)
-				} else if(!is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(lmmulti1.df, estimate_name = args$estimate_name) %>%
-						finalfit_merge(lmmulti2.df, estimate_name = args$estimate_name) %>%
-						finalfit_merge(lmmixed.df, estimate_name = args$estimate_name)
-				}
-			}
-		}
-		if (metrics == TRUE){
-			if (is.null(random_effect)){
-				if (is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(lmmulti1.df[[1]], estimate_name = args$estimate_name)
-					df.metrics = c(lmmulti1.df[[2]])
-				}else if(!is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(lmmulti1.df[[1]], estimate_name = args$estimate_name) %>%
-						finalfit_merge(lmmulti2.df[[1]], estimate_name = args$estimate_name)
-					df.metrics = c(lmmulti1.df[[2]],
-												 lmmulti2.df[[2]])
-				}
-			}else if(!is.null(random_effect)){
-				if (is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(lmmulti1.df[[1]], estimate_name = args$estimate_name) %>%
-						finalfit_merge(lmmixed.df[[1]], estimate_name = args$estimate_name)
-					df.metrics = c(lmmulti1.df[[2]],
-												 lmmixed.df[[2]])
-				} else if(!is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(lmmulti1.df[[1]], estimate_name = args$estimate_name) %>%
-						finalfit_merge(lmmulti2.df[[1]], estimate_name = args$estimate_name) %>%
-						finalfit_merge(lmmixed.df[[1]], estimate_name = args$estimate_name)
-					df.metrics = c(lmmulti1.df[[2]],
-												 lmmulti2.df[[2]],
-												 lmmixed.df[[2]])
+				} %>%
+				finalfit_merge(lmmulti.df, estimate_name = args$estimate_name)
+		} else {
+			df.out = df.out %>%
+				finalfit_merge(lmmulti1.df[[1]], estimate_name = args$estimate_name) %>%
+				{ if(!is.null(random_effect) & !is.null(explanatory_multi)){
+					finalfit_merge(., lmmulti2.df[[1]], estimate_name = args$estimate_name)
+				} else {
+					.
 				}
+				} %>% 
+				finalfit_merge(lmmulti.df[[1]], estimate_name = args$estimate_name)
+			
+			df.metrics = c(lmmulti1.df[[2]],
+										 lmmulti.df[[2]])
+			
+			if(!is.null(random_effect) & !is.null(explanatory_multi)){
+				df.metrics = c(lmmulti1.df[[2]],
+											 lmmulti2.df[[2]],
+											 lmmulti.df[[2]])
 			}
 		}
 	}
-
+	
 	# Label interactions
 	interaction_row = grep(":", df.out$fit_id)
 	df.out$label = as.character(df.out$label)
@@ -325,7 +308,7 @@ finalfit.lm = function(.data, dependent, explanatory, explanatory_multi=NULL, ra
 	df.out$label[interaction_row] = 	df.out$fit_id[interaction_row]
 	df.out$levels[interaction_row] = "Interaction"
 	df.out[interaction_row, 5] = "-"
-
+	
 	# Tidy up
 	index_fit_id = which(names(df.out)=="fit_id")
 	index_index = which(names(df.out)=="index")
@@ -334,7 +317,7 @@ finalfit.lm = function(.data, dependent, explanatory, explanatory_multi=NULL, ra
 	} else {
 		df.out = df.out[,-c(index_fit_id, index_index)]
 	}
-
+	
 	# Class
 	class(df.out) = c("data.frame.ff", class(df.out))
 	
@@ -343,9 +326,9 @@ finalfit.lm = function(.data, dependent, explanatory, explanatory_multi=NULL, ra
 		df.out = dependent_label(df.out=df.out, .data=.data, dependent=dependent,
 														 prefix=dependent_label_prefix, suffix = dependent_label_suffix)
 	}
-
+	
 	# Add metrics
-	if (metrics == TRUE){
+	if (metrics){
 		return(list(df.out, df.metrics))
 	} else {
 		return(df.out)
@@ -361,135 +344,104 @@ finalfit.lm = function(.data, dependent, explanatory, explanatory_multi=NULL, ra
 #' @rdname finalfit
 #' @export
 finalfit.glm = function(.data, dependent, explanatory, explanatory_multi=NULL, random_effect=NULL,
-												column = FALSE, keep_models=FALSE, metrics=FALSE,  add_dependent_label=TRUE,
+												model_args, column = FALSE, keep_models=FALSE, metrics=FALSE,  add_dependent_label=TRUE,
 												dependent_label_prefix="Dependent: ", dependent_label_suffix="", 
 												keep_fit_id=FALSE, ...){
-
+	
 	args = list(...)
-
+	model_args_in = model_args
+	
+	# Define arguments
+	model_args = c(list(.data=.data, dependent=dependent), model_args_in)
+	if(!is.null(random_effect)) model_args = c(model_args, list(random_effect=random_effect)) 
+	if(!is.null(explanatory_multi)){
+		model_args = c(model_args, list(explanatory=explanatory_multi))
+	} else {
+		model_args = c(model_args, list(explanatory=explanatory))
+	}
+	
 	# Defaults which can be modified via ...
 	if (is.null(args$estimate_name)) args$estimate_name = "OR"
-
+	
 	# Logistic regression ----
 	# Summary table
 	summary.out = summary_factorlist(.data, dependent, explanatory, p=FALSE, na_include=FALSE,
 																	 column=column, total_col=FALSE, orderbytotal=FALSE, fit_id=TRUE)
-
+	
 	# Univariable
-	glmuni.out = glmuni(.data, dependent, explanatory)
+	glmuni.out = do.call(glmuni, c(list(.data=.data, dependent=dependent, explanatory=explanatory), model_args_in))
 	glmuni.df = do.call(fit2df, c(list(.data=glmuni.out, estimate_suffix = " (univariable)"), args))
-
+	
 	# Merge dataframes
 	# Uni
 	df.out = finalfit_merge(summary.out, glmuni.df, estimate_name = args$estimate_name)
-
-	# This is standard approach, which is why written out in full but duplicated somewhat below
-	if(keep_models == FALSE){
-		# Multivariable/Mixed
-		if (is.null(random_effect)){
-			if (is.null(explanatory_multi)){
-				glmmulti.out = glmmulti(.data, dependent, explanatory)
-			} else {
-				glmmulti.out = glmmulti(.data, dependent, explanatory_multi)
-			}
-			glmmulti.df = do.call(fit2df,
-														c(list(.data=glmmulti.out, metrics=metrics, estimate_suffix = " (multivariable)"), args))
-		} else if (is.null(random_effect) == FALSE){
-			if (is.null(explanatory_multi)){
-				glmmulti.out = glmmixed(.data, dependent, explanatory, random_effect)
-			} else {
-				glmmulti.out = glmmixed(.data, dependent, explanatory_multi, random_effect)
-			}
-			glmmulti.df = do.call(fit2df,
-														c(list(.data=glmmulti.out, metrics=metrics, estimate_suffix = " (multilevel)"), args))
-		}
-
-		# Multi
-		if (metrics == FALSE){
+	
+	# Multivariable/Mixed
+	if(is.null(random_effect)){
+		glmmulti.out = do.call(glmmulti, model_args)
+		glmmulti.df = do.call(fit2df,
+													c(list(.data=glmmulti.out, metrics=metrics, estimate_suffix = " (multivariable)"), args))
+	} else if(!is.null(random_effect)){
+		glmmulti.out = do.call(glmmixed, model_args)
+		glmmulti.df = do.call(fit2df,
+													c(list(.data=glmmulti.out, metrics=metrics, estimate_suffix = " (multilevel)"), args))
+	}
+	
+	# Single multivariable model table
+	if(!keep_models){
+		if(!metrics){
 			df.out = finalfit_merge(df.out, glmmulti.df, estimate_name = args$estimate_name)
 		} else {
 			df.out = finalfit_merge(df.out, glmmulti.df[[1]], estimate_name = args$estimate_name)
 			df.metrics = glmmulti.df[[2]]
 		}
-
-	}else if(keep_models == TRUE){
-		glmmulti1.out = glmmulti(.data, dependent, explanatory)
+	} else if(keep_models){
+		
+		# Keep intermediate models
+		glmmulti1.out = do.call(glmmulti, c(list(.data=.data, dependent=dependent, explanatory=explanatory), model_args_in))
 		glmmulti1.df = do.call(fit2df,
-													 c(list(.data=glmmulti1.out, metrics=metrics,
-													 			 estimate_suffix = " (multivariable)"), args))
-		if (!is.null(explanatory_multi)){
-			glmmulti2.out = glmmulti(.data, dependent, explanatory_multi)
+													 c(list(.data=glmmulti1.out, metrics=metrics, estimate_suffix = " (multivariable full)"), args))
+		if(!is.null(random_effect) & !is.null(explanatory_multi)){
+			glmmulti2.out = do.call(
+				glmmixed, c(list(.data=.data, dependent=dependent, explanatory=explanatory, random_effect=random_effect), model_args_in))
 			glmmulti2.df = do.call(fit2df,
-														 c(list(.data=glmmulti2.out, metrics=metrics,
-														 			 estimate_suffix = " (multivariable reduced)"), args))
+														 c(list(.data=glmmulti2.out, metrics=metrics, estimate_suffix = " (multilevel full)"), args))
 		}
-		if (!is.null(random_effect)){
-			if (is.null(explanatory_multi)){
-				glmmixed.out = glmmixed(.data, dependent, explanatory, random_effect)
-			} else {
-				glmmixed.out = glmmixed(.data, dependent, explanatory_multi, random_effect)
-			}
-			glmmixed.df = do.call(fit2df,
-														c(list(.data=glmmixed.out, metrics=metrics,
-																	 estimate_suffix = " (multilevel)"), args))
-		}
-
-		if (metrics == FALSE){
-			if (is.null(random_effect)){
-				if (is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(glmmulti1.df, estimate_name = args$estimate_name)
-				}else if(!is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(glmmulti1.df, estimate_name = args$estimate_name) %>%
-						finalfit_merge(glmmulti2.df, estimate_name = args$estimate_name)
+		
+		if(!metrics){
+			df.out = df.out %>%
+				finalfit_merge(glmmulti1.df, estimate_name = args$estimate_name) %>%
+				{ if(!is.null(random_effect) & !is.null(explanatory_multi)){
+					finalfit_merge(., glmmulti2.df, estimate_name = args$estimate_name)
+				} else {
+					.
 				}
-			}else if(!is.null(random_effect)){
-				if (is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(glmmulti1.df, estimate_name = args$estimate_name) %>%
-						finalfit_merge(glmmixed.df, estimate_name = args$estimate_name)
-				} else if(!is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(glmmulti1.df, estimate_name = args$estimate_name) %>%
-						finalfit_merge(glmmulti2.df, estimate_name = args$estimate_name) %>%
-						finalfit_merge(glmmixed.df, estimate_name = args$estimate_name)
-				}
-			}
-		}
-		if (metrics == TRUE){
-			if (is.null(random_effect)){
-				if (is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(glmmulti1.df[[1]], estimate_name = args$estimate_name)
-					df.metrics = c(glmmulti1.df[[2]])
-				}else if(!is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(glmmulti1.df[[1]], estimate_name = args$estimate_name) %>%
-						finalfit_merge(glmmulti2.df[[1]], estimate_name = args$estimate_name)
-					df.metrics = c(glmmulti1.df[[2]],
-												 glmmulti2.df[[2]])
-				}
-			}else if(!is.null(random_effect)){
-				if (is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(glmmulti1.df[[1]], estimate_name = args$estimate_name) %>%
-						finalfit_merge(glmmixed.df[[1]], estimate_name = args$estimate_name)
-					df.metrics = c(glmmulti1.df[[2]],
-												 glmmixed.df[[2]])
-				} else if(!is.null(explanatory_multi)){
-					df.out = df.out %>%
-						finalfit_merge(glmmulti1.df[[1]], estimate_name = args$estimate_name) %>%
-						finalfit_merge(glmmulti2.df[[1]], estimate_name = args$estimate_name) %>%
-						finalfit_merge(glmmixed.df[[1]], estimate_name = args$estimate_name)
-					df.metrics = c(glmmulti1.df[[2]],
-												 glmmulti2.df[[2]],
-												 glmmixed.df[[2]])
+				} %>%
+				finalfit_merge(glmmulti.df, estimate_name = args$estimate_name)
+		} else {
+			df.out = df.out %>%
+				finalfit_merge(glmmulti1.df[[1]], estimate_name = args$estimate_name) %>%
+				{ if(!is.null(random_effect) & !is.null(explanatory_multi)){
+					finalfit_merge(., glmmulti2.df[[1]], estimate_name = args$estimate_name)
+				} else {
+					.
 				}
+				} %>% 
+				finalfit_merge(glmmulti.df[[1]], estimate_name = args$estimate_name)
+			
+			df.metrics = c(glmmulti1.df[[2]],
+										 glmmulti.df[[2]])
+			
+			if(!is.null(random_effect) & !is.null(explanatory_multi)){
+				df.metrics = c(glmmulti1.df[[2]],
+											 glmmulti2.df[[2]],
+											 glmmulti.df[[2]])
 			}
 		}
 	}
-
+	
+	
+	
 	# Label interactions
 	interaction_row = grep(":", df.out$fit_id)
 	df.out$label[interaction_row] = 	df.out$fit_id[interaction_row]
@@ -498,7 +450,7 @@ finalfit.glm = function(.data, dependent, explanatory, explanatory_multi=NULL, r
 	df.out[,5] = as.character(df.out[,5])
 	df.out[interaction_row, 4] = "-"
 	df.out[interaction_row, 5] = "-"
-
+	
 	# Tidy up
 	index_fit_id = which(names(df.out)=="fit_id")
 	index_index = which(names(df.out)=="index")
@@ -507,7 +459,7 @@ finalfit.glm = function(.data, dependent, explanatory, explanatory_multi=NULL, r
 	} else {
 		df.out = df.out[,-c(index_fit_id, index_index)]
 	}
-
+	
 	# Class
 	class(df.out) = c("data.frame.ff", class(df.out))
 	
@@ -516,7 +468,7 @@ finalfit.glm = function(.data, dependent, explanatory, explanatory_multi=NULL, r
 		df.out = dependent_label(df.out=df.out, .data=.data, dependent=dependent,
 														 prefix=dependent_label_prefix, suffix = dependent_label_suffix)
 	}
-
+	
 	# Add metrics
 	if (metrics){
 		return(list(df.out, df.metrics))
@@ -536,114 +488,82 @@ finalfit.glm = function(.data, dependent, explanatory, explanatory_multi=NULL, r
 #' @rdname finalfit
 #' @export
 finalfit.coxph = function(.data, dependent, explanatory, explanatory_multi=NULL, random_effect=NULL,
-													column = TRUE, keep_models=FALSE, metrics=FALSE, add_dependent_label=TRUE,
+													model_args, column = TRUE, keep_models=FALSE, metrics=FALSE, add_dependent_label=TRUE,
 													dependent_label_prefix="Dependent: ", dependent_label_suffix="", 
 													keep_fit_id=FALSE, ...){
-
+	
 	args = list(...)
-
+	model_args_in = model_args
+	
+	# Define arguments
+	model_args = c(list(.data=.data, dependent=dependent), model_args_in)
+	if(!is.null(random_effect)) model_args = c(model_args, list(random_effect=random_effect)) 
+	if(!is.null(explanatory_multi)){
+		model_args = c(model_args, list(explanatory=explanatory_multi))
+	} else {
+		model_args = c(model_args, list(explanatory=explanatory))
+	}
+	
 	# Defaults which can be modified via ...
 	if (is.null(args$estimate_name)) args$estimate_name = "HR"
-
+	
 	# No frailty
 	if(!is.null(random_effect)) stop("Random effects / frailty from package::coxme not currently implemented.
 																	 Consider adding `cluster(var)` to explanatory list for GEE equivalent.")
-
-	# Cox proprotional hazards model -----------------------------------------------------------
+	
+	# Cox proportional hazards model -----------------------------------------------------------
 	# Summary table
 	summary.out = suppressMessages(
 		suppressWarnings(
 			summary_factorlist(.data, dependent, explanatory, column = column, fit_id=TRUE)
 		))
 	
-	# Previous removed, but now include.
-	# summary.out = summary.out[,-3] # Remove 'all' column with total counts
-
 	# Univariable
-	coxphuni_out = coxphuni(.data, dependent, explanatory)
+	coxphuni_out = do.call(coxphuni, c(list(.data=.data, dependent=dependent, explanatory=explanatory), model_args_in))
 	coxphuni_df =	do.call(fit2df, c(list(.data=coxphuni_out, estimate_suffix = " (univariable)"), args))
-
+	
 	# Merge dataframes
 	# Uni
 	df.out = finalfit_merge(summary.out, coxphuni_df, estimate_name = args$estimate_name)
-
+	
 	# Multivariable
-	if(keep_models == FALSE){
-		if (is.null(explanatory_multi)){
-			coxphmulti.out = coxphmulti(.data, dependent, explanatory)
-		} else {
-			coxphmulti.out = coxphmulti(.data, dependent, explanatory_multi)
-		}
-		coxphmulti.df = do.call(fit2df,
-														c(list(.data=coxphmulti.out, estimate_suffix = " (multivariable)"),
-															metrics=metrics, args))
-
-
-		if (metrics == FALSE){
+	coxphmulti.out = do.call(coxphmulti, model_args)
+	coxphmulti.df = do.call(fit2df,
+													c(list(.data=coxphmulti.out, estimate_suffix = " (multivariable)"),
+														metrics=metrics, args))
+	if(!keep_models){
+		if (!metrics){
 			df.out = finalfit_merge(df.out, coxphmulti.df, estimate_name = args$estimate_name)
 		} else {
 			df.out = finalfit_merge(df.out, coxphmulti.df[[1]], estimate_name = args$estimate_name)
 			df.metrics = coxphmulti.df[[2]]
 		}
-
-
-	}else if(keep_models == TRUE){
-		coxphmulti1.out = coxphmulti(.data, dependent, explanatory)
+		
+		
+	} else if (keep_models){
+		coxphmulti1.out = do.call(coxphmulti, c(list(.data=.data, dependent=dependent, explanatory=explanatory), model_args_in))
 		coxphmulti1.df = do.call(fit2df,
 														 c(list(.data=coxphmulti1.out, metrics=metrics,
-														 			 estimate_suffix = " (multivariable)"), args))
-		if (!is.null(explanatory_multi)){
-			coxphmulti2.out = coxphmulti(.data, dependent, explanatory_multi)
-			coxphmulti2.df = do.call(fit2df,
-															 c(list(.data=coxphmulti2.out, metrics=metrics,
-															 			 estimate_suffix = " (multivariable reduced)"), args))
-		}
-
-		if (metrics == FALSE){
-			if (is.null(explanatory_multi)){
-				df.out = df.out %>%
-					finalfit_merge(coxphmulti1.df, estimate_name = args$estimate_name)
-			}else if(!is.null(explanatory_multi)){
-				df.out = df.out %>%
-					finalfit_merge(coxphmulti1.df, estimate_name = args$estimate_name) %>%
-					finalfit_merge(coxphmulti2.df, estimate_name = args$estimate_name)
-			}
-		}
-
-		if (metrics == TRUE){
-			if (is.null(explanatory_multi)){
-				df.out = df.out %>%
-					finalfit_merge(coxphmulti1.df[[1]], estimate_name = args$estimate_name)
-				df.metrics = c(coxphmulti1.df[[2]])
-			}else if(!is.null(explanatory_multi)){
-				df.out = df.out %>%
-					finalfit_merge(coxphmulti1.df[[1]], estimate_name = args$estimate_name) %>%
-					finalfit_merge(coxphmulti2.df[[1]], estimate_name = args$estimate_name)
-				df.metrics = c(coxphmulti1.df[[2]],
-											 coxphmulti2.df[[2]])
-			}
+														 			 estimate_suffix = " (multivariable full)"), args))
+		if(!metrics){
+			df.out = df.out %>%
+				finalfit_merge(coxphmulti1.df, estimate_name = args$estimate_name) %>%
+				finalfit_merge(coxphmulti.df, estimate_name = args$estimate_name)
+		} else {
+			df.out = df.out %>%
+				finalfit_merge(coxphmulti1.df[[1]], estimate_name = args$estimate_name) %>%
+				finalfit_merge(coxphmulti.df[[1]], estimate_name = args$estimate_name)
+			
+			df.metrics = c(coxphmulti1.df[[2]],
+										 coxphmulti.df[[2]])
 		}
 	}
-
-	# # Multi
-	## Add frailty later
-	# if (metrics == FALSE){
-	# 	df.out = finalfit_merge(df.out, glmmulti_df)
-	# } else {
-	# 	df.out = finalfit_merge(df.out, glmmulti_df[[1]])
-	# }
-	#
-	# if (is.null(random_effect)){
-	# 	names(df.out)[which(names(df.out)=="OR")] = "OR (multivariable)"
-	# } else {
-	# 	names(df.out)[which(names(df.out)=="OR")] = "OR (multilevel)"
-	# }
-
+	
 	# Label interactions
 	interaction_row = grep(":", df.out$fit_id)
 	df.out$label[interaction_row] = 	df.out$fit_id[interaction_row]
 	df.out$levels[interaction_row] = "Interaction"
-
+	
 	# Tidy up
 	index_fit_id = which(names(df.out)=="fit_id")
 	index_index = which(names(df.out)=="index")
@@ -655,15 +575,15 @@ finalfit.coxph = function(.data, dependent, explanatory, explanatory_multi=NULL,
 	
 	# Class
 	class(df.out) = c("data.frame.ff", class(df.out))
-
+	
 	# Add dependent name label
 	if(add_dependent_label){
 		df.out = dependent_label(df.out=df.out, .data=.data, dependent=dependent,
 														 prefix=dependent_label_prefix, suffix = dependent_label_suffix)
 	}
-
+	
 	# Add metrics
-	if (metrics == TRUE){
+	if (metrics){
 		return(list(df.out, df.metrics))
 	} else {
 		return(df.out)
